@@ -141,6 +141,7 @@ fo_context_init(TALLOC_CTX *mem_ctx, struct fo_options *opts)
     ctx->opts->srv_retry_timeout = opts->srv_retry_timeout;
     ctx->opts->retry_timeout = opts->retry_timeout;
     ctx->opts->family_order  = opts->family_order;
+    ctx->opts->service_resolv_timeout = opts->service_resolv_timeout;
 
     DEBUG(3, ("Created new fail over context, retry timeout is %d\n",
               ctx->opts->retry_timeout));
@@ -224,7 +225,8 @@ collapse_srv_lookup(struct fo_server *server)
     struct fo_server *tmp, *meta;
 
     meta = server->srv_data->meta;
-    DEBUG(4, ("Need to refresh SRV lookup for domain %s\n", meta->srv_data->dns_domain))
+    DEBUG(4, ("Need to refresh SRV lookup for domain %s\n",
+              meta->srv_data->dns_domain));
 
     if (server != meta) {
         while (server->prev && server->prev->srv_data == meta->srv_data) {
@@ -734,10 +736,13 @@ struct resolve_service_state {
 
     struct resolv_ctx *resolv;
     struct tevent_context *ev;
+    struct tevent_timer *timeout_handler;
     struct fo_ctx *fo_ctx;
 };
 
 
+static errno_t fo_resolve_service_activate_timeout(struct tevent_req *req,
+            struct tevent_context *ev, const unsigned long timeout_seconds);
 static void fo_resolve_service_cont(struct tevent_req *subreq);
 static void fo_resolve_service_done(struct tevent_req *subreq);
 static bool fo_resolve_service_server(struct tevent_req *req);
@@ -776,6 +781,14 @@ fo_resolve_service_send(TALLOC_CTX *mem_ctx, struct tevent_context *ev,
         goto done;
     }
 
+    /* Activate per-service timeout handler */
+    ret = fo_resolve_service_activate_timeout(req, ev,
+                                        ctx->opts->service_resolv_timeout);
+    if (ret != EOK) {
+        DEBUG(SSSDBG_OP_FAILURE, ("Could not set service timeout\n"));
+        goto done;
+    }
+
     if (fo_is_srv_lookup(server)) {
         /* Don't know the server yet, must do a SRV lookup */
         subreq = resolve_srv_send(state, ev, resolv,
@@ -809,6 +822,40 @@ done:
 static void set_server_common_status(struct server_common *common,
                                      enum server_status status);
 
+static void
+fo_resolve_service_timeout(struct tevent_context *ev,
+                           struct tevent_timer *te,
+                           struct timeval tv, void *pvt)
+{
+    struct tevent_req *req = talloc_get_type(pvt, struct tevent_req);
+
+    DEBUG(SSSDBG_MINOR_FAILURE, ("Service resolving timeout reached\n"));
+    tevent_req_error(req, ETIMEDOUT);
+}
+
+static errno_t
+fo_resolve_service_activate_timeout(struct tevent_req *req,
+                                    struct tevent_context *ev,
+                                    const unsigned long timeout_seconds)
+{
+    struct timeval tv;
+    struct resolve_service_state *state = tevent_req_data(req,
+                                        struct resolve_service_state);
+
+    tv = tevent_timeval_current();
+    tv = tevent_timeval_add(&tv, timeout_seconds, 0);
+    state->timeout_handler = tevent_add_timer(ev, state, tv,
+                                           fo_resolve_service_timeout, req);
+    if (state->timeout_handler == NULL) {
+        DEBUG(SSSDBG_CRIT_FAILURE, ("tevent_add_timer failed.\n"));
+        return ENOMEM;
+    }
+
+    DEBUG(SSSDBG_TRACE_INTERNAL, ("Resolve timeout set to %lu seconds\n",
+          timeout_seconds));
+    return EOK;
+}
+
 /* SRV resolving finished, see if we got server to work with */
 static void
 fo_resolve_service_cont(struct tevent_req *subreq)
@@ -849,7 +896,8 @@ fo_resolve_service_server(struct tevent_req *req)
             tevent_req_error(req, ENOMEM);
             return true;
         }
-        tevent_req_set_callback(subreq, fo_resolve_service_done, req);
+        tevent_req_set_callback(subreq, fo_resolve_service_done,
+                                state->server->common);
         fo_set_server_status(state->server, SERVER_RESOLVING_NAME);
         /* FALLTHROUGH */
     case SERVER_RESOLVING_NAME:
@@ -872,34 +920,30 @@ fo_resolve_service_server(struct tevent_req *req)
 static void
 fo_resolve_service_done(struct tevent_req *subreq)
 {
-    struct tevent_req *req = tevent_req_callback_data(subreq,
-                                                      struct tevent_req);
-    struct resolve_service_state *state = tevent_req_data(req,
-                                                struct resolve_service_state);
-    struct server_common *common;
+    struct server_common *common = tevent_req_callback_data(subreq,
+                                                        struct server_common);
     int resolv_status;
     struct resolve_service_request *request;
     int ret;
 
-    if (state->server->common->rhostent != NULL) {
-        talloc_zfree(state->server->common->rhostent);
+    if (common->rhostent != NULL) {
+        talloc_zfree(common->rhostent);
     }
 
-    ret = resolv_gethostbyname_recv(subreq, state->server->common,
+    ret = resolv_gethostbyname_recv(subreq, common,
                                     &resolv_status, NULL,
-                                    &state->server->common->rhostent);
+                                    &common->rhostent);
     talloc_zfree(subreq);
     if (ret != EOK) {
         DEBUG(1, ("Failed to resolve server '%s': %s\n",
-                  state->server->common->name,
+                  common->name,
                   resolv_strerror(resolv_status)));
-        set_server_common_status(state->server->common, SERVER_NOT_WORKING);
+        set_server_common_status(common, SERVER_NOT_WORKING);
     } else {
-        set_server_common_status(state->server->common, SERVER_NAME_RESOLVED);
+        set_server_common_status(common, SERVER_NAME_RESOLVED);
     }
 
     /* Take care of all requests for this server. */
-    common = state->server->common; /* state can disappear now */
     while ((request = common->request_list) != NULL) {
         DLIST_REMOVE(common->request_list, request);
         if (resolv_status) {
@@ -1432,3 +1476,10 @@ void fo_reset_services(struct fo_ctx *fo_ctx)
     }
 }
 
+struct fo_service *
+fo_get_server_service(struct fo_server *server)
+{
+    if (!server) return NULL;
+    return server->service;
+}
+
