@@ -1388,7 +1388,8 @@ int sdap_service_init(TALLOC_CTX *memctx, struct be_ctx *ctx,
                 goto done;
             }
 
-            ret = be_fo_add_srv_server(ctx, service_name, dns_service_name,
+            ret = be_fo_add_srv_server(ctx, service_name,
+                                       dns_service_name, NULL,
                                        BE_FO_PROTO_TCP, false, srv_user_data);
             if (ret) {
                 DEBUG(0, ("Failed to add server\n"));
