@@ -101,11 +101,6 @@ int sssm_proxy_id_init(struct be_ctx *bectx,
     }
     ctx->be = bectx;
 
-    ret = confdb_get_int(bectx->cdb, ctx, bectx->conf_path,
-                         CONFDB_DOMAIN_ENTRY_CACHE_TIMEOUT, 600,
-                         &ctx->entry_cache_timeout);
-    if (ret != EOK) goto done;
-
     ret = confdb_get_string(bectx->cdb, ctx, bectx->conf_path,
                             CONFDB_PROXY_LIBNAME, NULL, &libname);
     if (ret != EOK) goto done;
@@ -237,6 +232,61 @@ int sssm_proxy_id_init(struct be_ctx *bectx,
                                                                dlerror()));
     }
 
+    ctx->ops.getservbyname_r = proxy_dlsym(ctx->handle,
+                                           "_nss_%s_getservbyname_r",
+                                           libname);
+    if (!ctx->ops.getservbyname_r) {
+        DEBUG(SSSDBG_MINOR_FAILURE,
+              ("Failed to load _nss_%s_getservbyname_r, error: %s. "
+               "The library does not support services.\n",
+               libname,
+               dlerror()));
+    }
+
+    ctx->ops.getservbyport_r = proxy_dlsym(ctx->handle,
+                                           "_nss_%s_getservbyport_r",
+                                           libname);
+    if (!ctx->ops.getservbyport_r) {
+        DEBUG(SSSDBG_MINOR_FAILURE,
+              ("Failed to load _nss_%s_getservbyport_r, error: %s. "
+               "The library does not support services.\n",
+               libname,
+               dlerror()));
+    }
+
+    ctx->ops.setservent = proxy_dlsym(ctx->handle,
+                                      "_nss_%s_setservent",
+                                      libname);
+    if (!ctx->ops.setservent) {
+        DEBUG(SSSDBG_MINOR_FAILURE,
+              ("Failed to load _nss_%s_setservent, error: %s. "
+               "The library does not support services.\n",
+               libname,
+               dlerror()));
+    }
+
+    ctx->ops.getservent_r = proxy_dlsym(ctx->handle,
+                                        "_nss_%s_getservent_r",
+                                        libname);
+    if (!ctx->ops.getservent_r) {
+        DEBUG(SSSDBG_MINOR_FAILURE,
+              ("Failed to load _nss_%s_getservent_r, error: %s. "
+               "The library does not support services.\n",
+               libname,
+               dlerror()));
+    }
+
+    ctx->ops.endservent = proxy_dlsym(ctx->handle,
+                                      "_nss_%s_endservent",
+                                      libname);
+    if (!ctx->ops.endservent) {
+        DEBUG(SSSDBG_MINOR_FAILURE,
+              ("Failed to load _nss_%s_endservent, error: %s. "
+               "The library does not support services.\n",
+               libname,
+               dlerror()));
+    }
+
     *ops = &proxy_id_ops;
     *pvt_data = ctx;
     ret = EOK;
