@@ -30,7 +30,7 @@
 
 int main(int argc, const char **argv)
 {
-    TALLOC_CTX *mem_ctx;
+    TALLOC_CTX *mem_ctx = NULL;
     int pc_debug = SSSDBG_DEFAULT;
     const char *pc_domain = NULL;
     const char *pc_user = NULL;
