@@ -297,8 +297,7 @@ int group_show(TALLOC_CTX *mem_ctx,
     int i;
 
     /* First, search for the root group */
-    ret = sysdb_search_group_by_name(mem_ctx, sysdb,
-                                     domain, name, attrs, &msg);
+    ret = sysdb_search_group_by_name(mem_ctx, sysdb, name, attrs, &msg);
     if (ret) {
         DEBUG(2, ("Search failed: %s (%d)\n", strerror(ret), ret));
         goto done;
@@ -402,9 +401,7 @@ static int group_show_trim_memberof(TALLOC_CTX *mem_ctx,
             return ENOMEM;
         }
 
-        ret = sysdb_search_groups(mem_ctx, sysdb,
-                                  domain, filter, NULL,
-                                  &count, &msgs);
+        ret = sysdb_search_groups(mem_ctx, sysdb, filter, NULL, &count, &msgs);
         /* ENOENT is OK, the group is just not a direct parent */
         if (ret != EOK && ret != ENOENT) {
             return ret;
@@ -472,8 +469,7 @@ int group_show_recurse(TALLOC_CTX *mem_ctx,
             continue;
         }
 
-        ret = sysdb_search_group_by_name(mem_ctx, sysdb,
-                                         domain, group_members[i],
+        ret = sysdb_search_group_by_name(mem_ctx, sysdb, group_members[i],
                                          attrs, &msg);
         if (ret) {
             DEBUG(2, ("Search failed: %s (%d)\n", strerror(ret), ret));
@@ -527,8 +523,7 @@ static int group_show_mpg(TALLOC_CTX *mem_ctx,
         goto fail;
     }
 
-    ret = sysdb_search_user_by_name(info, sysdb,
-                                    domain, name, attrs, &msg);
+    ret = sysdb_search_user_by_name(info, sysdb, name, attrs, &msg);
     if (ret) {
         DEBUG(2, ("Search failed: %s (%d)\n", strerror(ret), ret));
         goto fail;
@@ -606,7 +601,7 @@ static void print_recursive(struct group_info **group_members, int level)
 int main(int argc, const char **argv)
 {
     int ret = EXIT_SUCCESS;
-    int pc_debug = 0;
+    int pc_debug = SSSDBG_DEFAULT;
     bool pc_recursive = false;
     const char *pc_groupname = NULL;
     struct tools_ctx *tctx = NULL;
@@ -644,7 +639,7 @@ int main(int argc, const char **argv)
         }
     }
 
-    debug_level = pc_debug;
+    debug_level = debug_convert_old_level(pc_debug);
 
     if (ret != -1) {
         BAD_POPT_PARAMS(pc, poptStrerror(ret), ret, fini);
